# Copyright (c) 2014 Thomas Heller
# Copyright (c) 2015 Andreas Schaefer
#
# SPDX-License-Identifier: BSL-1.0
# Distributed under the Boost Software License, Version 1.0. (See accompanying
# file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

set(HPX_FIND_PACKAGE ON)
cmake_policy(VERSION 3.13)

# Forward HPX_* cache variables
include("${CMAKE_CURRENT_LIST_DIR}/HPXCacheVariables.cmake")

# include exported targets
include("${CMAKE_CURRENT_LIST_DIR}/HPXInternalTargets.cmake")
include("${CMAKE_CURRENT_LIST_DIR}/HPXTargets.cmake")

# include HPX cmake utilities
include("${CMAKE_CURRENT_LIST_DIR}/HPXMacros.cmake")

get_filename_component(_hpx_root_dir "${CMAKE_CURRENT_LIST_DIR}/../../.." ABSOLUTE)

set(HPX_VERSION_STRING "1.6.0")
set(HPX_VERSION_MAJOR 1)
set(HPX_VERSION_MINOR 6)
set(HPX_VERSION_SUBMINOR 0)

set(HPX_PREFIX "${_hpx_root_dir}")
set(HPX_DEBUG_POSTFIX "d")
set(HPX_BUILD_TYPE "Release")
set(HPX_CXX_STANDARD "14")

set(HPX_GIT_COMMIT "unknown" CACHE STRING "Revision of HPX from Git" FORCE)

set(HPX_CXX_COMPILER "/usr/lib/mpich/bin/mpicxx" CACHE STRING "CXX compiler for HPX" FORCE)
set(HPX_CXX_COMPILER_ID "GNU" CACHE STRING "CXX compiler id for HPX" FORCE)
set(HPX_CXX_COMPILER_VERSION "11.0.0" CACHE STRING "CXX compiler version for HPX" FORCE)

##################################################
# Setup the imported libraries (publicly linked) #

# Allocator
set(HPX_JEMALLOC_ROOT "")
set(HPX_TCMALLOC_ROOT "/usr")
set(HPX_TBBMALLOC_ROOT "")
# Special handle for mimalloc cause we can't specify HPX_MIMALLOC_ROOT as a HINT
# to find_package
set(HPX_MIMALLOC_ROOT "")
if (NOT MIMALLOC_ROOT AND NOT "$ENV{MIMALLOC_ROOT}")
  set(MIMALLOC_ROOT ${HPX_MIMALLOC_ROOT})
endif()
include(HPX_SetupAllocator)

# Boost
# Separate boost targets to be unarily linked to some modules
set(HPX_BOOST_ROOT "/usr")
# By default BOOST_ROOT is set to HPX_BOOST_ROOT (not necessary for PAPI or
# HWLOC cause we are specifying HPX_<lib>_ROOT as an HINT to find_package)
if (NOT BOOST_ROOT AND NOT "$ENV{BOOST_ROOT}")
  set(BOOST_ROOT ${HPX_BOOST_ROOT})
endif()
include(HPX_SetupBoost)
include(HPX_SetupBoostFilesystem)
include(HPX_SetupBoostProgramOptions)

# HIP
include(HPX_SetupHIP)

# Hwloc
set(HPX_HWLOC_ROOT "/usr")
include(HPX_SetupHwloc)

# Papi
set(HPX_PAPI_ROOT "")
include(HPX_SetupPapi)

# CUDA
include(HPX_SetupCUDA)

# APEX
set(APEX_WITH_MSR "")
set(MSR_ROOT "")
set(APEX_WITH_ACTIVEHARMONY "")
set(ACTIVEHARMONY_ROOT "")
set(APEX_WITH_OTF2 "")
set(OTF2_ROOT "")
include(HPX_SetupApex)
##################################################

set(HPX_WITH_MALLOC_DEFAULT tcmalloc)

if(HPX_WITH_DATAPAR_VC AND NOT Vc_DIR)
  set(Vc_DIR "")
endif()
include(HPX_SetupVc)

if(NOT HPX_CMAKE_LOGLEVEL)
  set(HPX_CMAKE_LOGLEVEL "WARN")
endif()

hpx_check_cmake_build_type()
hpx_check_compiler_compatibility()
hpx_check_boost_compatibility()
hpx_check_allocator_compatibility()

if(NOT DEFINED ${CMAKE_FIND_PACKAGE_NAME}_FOUND)
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND true)
endif()

# Set legacy variables for linking and include directories
set(HPX_LIBRARIES "HPX::hpx;")
# All properties are propagated from HPX::hpx, so the following can be empty
set(HPX_INCLUDE_DIRS "")
set(HPX_LIBRARY_DIR "")
set(HPX_LINK_LIBRARIES "")
set(HPX_LINKER_FLAGS "")
